import type { ThemedToken, ThemeRegistration } from 'shiki'
import type { SyntaxHighlighterProps } from 'src/types/shiki'
import React, { useMemo } from 'react'
import { Platform, ScrollView, StyleSheet, Text, View } from 'react-native'
import { FontStyle } from 'shiki'
import { useSyntaxHighlighter } from '../../hooks/useSyntaxHighlighter'
import { getRNStylesFromShikiStyle } from '../../utils/style-transformer'

const monospaceFont = Platform.select({
  ios: 'Menlo',
  android: 'monospace',
  default: 'monospace',
})

const baseStyles = StyleSheet.create({
  scrollView: {
    flex: 1,
    minHeight: 20,
    backgroundColor: '#1e1e1e',
  },
  contentContainer: {
    flexGrow: 1,
    minWidth: '100%',
  },
  codeContainer: {
    paddingVertical: 8,
    minWidth: '100%',
  },
  lineContainer: {
    flexDirection: 'row',
    minWidth: '100%',
    paddingHorizontal: 16,
    paddingVertical: 1,
  },
  line: {
    flexDirection: 'row',
    flexWrap: 'wrap',
    flex: 1,
  },
  token: {
    fontFamily: monospaceFont,
    fontSize: 14,
    color: '#d4d4d4',
  },
})

interface TokenStyle {
  color?: string
  fontStyle?: 'normal' | 'italic'
  fontWeight?: 'normal' | 'bold'
}

function getFontStyle(style: number | undefined): 'normal' | 'italic' {
  return style === FontStyle.Italic ? 'italic' : 'normal'
}

function getFontWeight(style: number | undefined): 'normal' | 'bold' {
  return style === FontStyle.Bold ? 'bold' : 'normal'
}

const SyntaxHighlighter: React.FC<SyntaxHighlighterProps> = ({ text, language, languageId, theme, fontSize = 14 }) => {
  const { tokens } = useSyntaxHighlighter({
    text,
    language,
    languageId: languageId || String(language),
    theme,
  })

  const themeStyles = useMemo(() => getRNStylesFromShikiStyle(theme as ThemeRegistration), [theme])

  const renderToken = (token: ThemedToken, index: number) => {
    const tokenStyle: TokenStyle = {
      color: token.color,
      fontStyle: getFontStyle(token.fontStyle),
      fontWeight: getFontWeight(token.fontStyle),
    }

    return (
      <Text key={index} style={[baseStyles.token, tokenStyle, { fontSize }]}>
        {token.content}
      </Text>
    )
  }

  return (
    <ScrollView horizontal style={[baseStyles.scrollView, themeStyles.editor]}>
      <View style={baseStyles.codeContainer}>
        {tokens.map((line, i) => (
          <View key={i} style={baseStyles.line}>
            {line.map((token, j) => renderToken(token, j))}
          </View>
        ))}
      </View>
    </ScrollView>
  )
}

export default SyntaxHighlighter
